@@ -3,33 +3,32 @@
  *
  *  Copyright (C) 2001, 2002 Michael H. Schimek
  *
- *  Based on code from AleVT 1.5.1
- *  Copyright (C) 1998, 1999 Edgar Toernig <froese@gmx.de>
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: exp-txt.c,v 1.15 2006/02/10 06:25:37 mschimek Exp $ */
+/* $Id: exp-txt.c,v 1.23 2008/02/24 14:17:41 mschimek Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <limits.h>
 #include <iconv.h>
-#include <assert.h>
 
+#include "misc.h"
 #include "lang.h"
 #include "export.h"
 #include "exp-txt.h"
@@ -97,15 +96,16 @@ iconv_formats[] = {
 };
 
 static const char *
-color_names[] __attribute__ ((unused)) = {
+color_names[] _vbi_unused = {
 	N_("Black"), N_("Red"), N_("Green"), N_("Yellow"),
 	N_("Blue"), N_("Magenta"), N_("Cyan"), N_("White"),
 	N_("Any")
 };
 
 static const char *
-terminal[] __attribute__ ((unused)) = {
-	/* TRANSLATORS: Terminal control codes */
+terminal[] = {
+	/* TRANSLATORS:
+	   Terminal control codes. */
 	N_("None"), N_("ANSI X3.64 / VT 100"), N_("VT 200")
 };
 
@@ -261,9 +261,9 @@ match_color8(vbi_rgba color)
 	int i, d, imin = 0, dmin = INT_MAX;
 
 	for (i = 0; i < 8; i++) {
-		d  = ABS(       (i & 1) * 0xFF - VBI_R(color));
-		d += ABS(((i >> 1) & 1) * 0xFF - VBI_G(color));
-		d += ABS( (i >> 2)      * 0xFF - VBI_B(color));
+		d  = ABS((int)(       (i & 1) * 0xFF - VBI_R(color)));
+		d += ABS((int)(((i >> 1) & 1) * 0xFF - VBI_G(color)));
+		d += ABS((int)( (i >> 2)      * 0xFF - VBI_B(color)));
 
 		if (d < dmin) {
 			dmin = d;
@@ -560,12 +560,13 @@ print_char(text_instance *text, int endian, vbi_page *pg, vbi_char old, vbi_char
 			p = _stpcpy(p, "5;"); /* flash */
 		}
 
+		/* FIXME what is the real buffer size? */
 		if (chg.foreground)
-			p += sprintf(p, "3%c;", '0'
+			p += snprintf(p, 4, "3%c;", '0'
 				     + match_color8(pg->color_map[this.foreground]));
 
 		if (chg.background)
-			p += sprintf(p, "4%c;", '0'
+			p += snprintf(p, 4, "4%c;", '0'
 				     + match_color8(pg->color_map[this.background]));
 
 		if (p[-1] == '[')
@@ -591,7 +592,8 @@ print_char(text_instance *text, int endian, vbi_page *pg, vbi_char old, vbi_char
 }
 
 static vbi_bool
-export(vbi_export *e, FILE *fp, vbi_page *pg)
+export				(vbi_export *		e,
+				 vbi_page *		pg)
 {
 	int endian = vbi_ucs2be();
 	text_instance *text = PARENT(e, text_instance, export);
@@ -605,10 +607,16 @@ export(vbi_export *e, FILE *fp, vbi_page *pg)
 	else
 		charset = iconv_formats[text->format];
 
-	if ((text->cd = iconv_open(charset, "UCS-2")) == (iconv_t) -1 || endian < 0) {
+	text->cd = iconv_open (charset, "UCS-2");
+	if ((iconv_t) -1 == text->cd || endian < 0) {
 		vbi_export_error_printf(&text->export,
-			_("Character conversion Unicode (UCS-2) to %s not supported."),
+					_("Character conversion Unicode "
+					  "(UCS-2) to %s not supported."),
 			charset);
+
+		if ((iconv_t) -1 != text->cd)
+			iconv_close (text->cd);
+
 		return FALSE;
 	}
 
@@ -628,9 +636,9 @@ export(vbi_export *e, FILE *fp, vbi_page *pg)
 				iconv_close(text->cd);
 				return FALSE;
 			} else if (n == 1) {
-				fputc(text->buf[0], fp);
+				vbi_export_putc (e, text->buf[0]);
 			} else {
-				fwrite(text->buf, 1, n, fp);
+				vbi_export_write (e, text->buf, n);
 			}
 
 			old = *cp++;
@@ -640,17 +648,18 @@ export(vbi_export *e, FILE *fp, vbi_page *pg)
 
 		if (row >= pg->rows) {
 			if (text->term > 0)
-				fprintf(fp, "\e[m\n"); /* reset */
+				vbi_export_printf (e, "\e[m\n"); /* reset */
 			else
-				fputc('\n', fp);
+				vbi_export_putc (e, '\n');
 			break;
-		} else
-			fputc('\n', fp);
+		} else {
+			vbi_export_putc (e, '\n');
+		}
 	}
 
 	iconv_close(text->cd);
 
-	return !ferror(fp);
+	return !e->write_error;
 }
 
 static vbi_export_info
@@ -673,3 +682,10 @@ vbi_export_class_text = {
 	.option_set		= option_set,
 	.export			= export
 };
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
