@@ -1,22 +1,25 @@
 /*
- *  Basic I/O between VBI proxy client & server
+ *  libvbi -- Basic I/O between VBI proxy client & server
  *
- *  Copyright (C) 2003,2004 Tom Zoerner
+ *  Copyright (C) 2003, 2004 Tom Zoerner
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
+ */
+
+/*
  *  Description:
  *
  *    This module contains a collection of functions for lower-level
@@ -28,10 +31,31 @@
  *
  *    Both UNIX domain and IPv4 and IPv6 sockets are implemented, but
  *    the latter ones are currently not officially supported.
- *
- *  $Id: proxy-msg.c,v 1.15 2006/05/22 09:08:46 mschimek Exp $
+ */
+ 
+/*
+ *  $Id: proxy-msg.c,v 1.20 2008/02/19 00:35:21 mschimek Exp $
  *
  *  $Log: proxy-msg.c,v $
+ *  Revision 1.20  2008/02/19 00:35:21  mschimek
+ *  *** empty log message ***
+ *
+ *  Revision 1.19  2007/11/27 18:31:07  mschimek
+ *  Updated the FSF address in the copyright notice.
+ *
+ *  Revision 1.18  2007/08/27 10:17:50  mschimek
+ *  *** empty log message ***
+ *
+ *  Revision 1.17  2007/08/27 06:44:40  mschimek
+ *  vbi_proxy_msg_get_local_socket_addr, vbi_proxy_msg_accept_connection,
+ *  vbi_proxy_msg_resolve_symlinks: Replaced strncpy() by the faster a
+ *  safer strlcpy().
+ *  vbi_proxy_msg_logger, vbi_proxy_msg_accept_connection: Replaced
+ *  sprintf() by the safer snprintf().
+ *
+ *  Revision 1.16  2007/07/23 20:01:18  mschimek
+ *  *** empty log message ***
+ *
  *  Revision 1.15  2006/05/22 09:08:46  mschimek
  *  *** empty log message ***
  *
@@ -116,13 +140,20 @@
 #include "misc.h"
 #include "proxy-msg.h"
 
+#ifdef ENABLE_V4L2
+#include <asm/types.h>		/* __u8 and friends for videodev2k.h */
+#ifndef HAVE_S64_U64
+#  include <inttypes.h>
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+#include "videodev2k.h"
+#endif
 #ifdef ENABLE_V4L
 #include "videodev.h"
 #endif
-#ifdef ENABLE_V4L2
-#include <asm/types.h>
-#include "videodev2k.h"
-#endif
 
 #define dprintf1(fmt, arg...)    do {if (proxy_msg_trace >= 1) fprintf(stderr, "proxy_msg: " fmt, ## arg);} while(0)
 #define dprintf2(fmt, arg...)    do {if (proxy_msg_trace >= 2) fprintf(stderr, "proxy_msg: " fmt, ## arg);} while(0)
@@ -183,10 +214,10 @@ void vbi_proxy_msg_logger( int level, int clnt_fd, int errCode, const char * pTe
       memset(argv, 0, sizeof(argv));
       /* add pointer to file descriptor (for client requests) or pid (for general infos) */
       if (clnt_fd != -1)
-         sprintf(fdstr, "fd %d: ", clnt_fd);
+         snprintf(fdstr, sizeof (fdstr), "fd %d: ", clnt_fd);
       else
       {
-         sprintf(fdstr, "pid %d: ", (int)getpid());
+         snprintf(fdstr, sizeof (fdstr), "pid %d: ", (int)getpid());
       }
       argv[argc++] = fdstr;
 
@@ -542,7 +573,8 @@ vbi_bool vbi_proxy_msg_handle_read( VBIPROXY_MSG_STATE * pIO,
          }
          else if ((len < 0) && (errno != EAGAIN) && (errno != EINTR))
          {  /* network error -> close the connection */
-            dprintf1("handle_io: read error on fd %d: len=%d, %s\n", pIO->sock_fd, len, strerror(errno));
+   	    dprintf1("handle_io: read error on fd %d: len=%ld, %s\n",
+		     pIO->sock_fd, (long) len, strerror(errno));
             result = FALSE;
          }
          else if (errno == EAGAIN)
@@ -760,7 +792,7 @@ static int vbi_proxy_msg_get_local_socket_addr( const char * pPathName, const st
       res->ai_addr      = (struct sockaddr *) saddr;
       res->ai_addrlen   = sizeof(struct sockaddr_un);
 
-      strncpy(saddr->sun_path, pPathName, sizeof(saddr->sun_path) - 1);
+      strlcpy(saddr->sun_path, pPathName, sizeof(saddr->sun_path) - 1);
       saddr->sun_path[sizeof(saddr->sun_path) - 1] = 0;
       saddr->sun_family = AF_UNIX;
 
@@ -926,7 +958,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
                hent = gethostbyaddr((void *) &peerAddr.sa, maxLength, AF_INET);
                if (hent != NULL)
                {
-                  strncpy(hname_buf, hent->h_name, sizeof(hname_buf) -1);
+                  strlcpy(hname_buf, hent->h_name, sizeof(hname_buf) -1);
                   hname_buf[sizeof(hname_buf) - 1] = 0;
                }
                else
@@ -934,7 +966,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
 		  struct sockaddr_in *sa;
 
 		  sa = (struct sockaddr_in *) &peerAddr.sa;
-                  sprintf(hname_buf, "%s, port %d", inet_ntoa(sa->sin_addr), sa->sin_port);
+                  snprintf(hname_buf, sizeof (hname_buf), "%s, port %d", inet_ntoa(sa->sin_addr), sa->sin_port);
 	       }
 
                vbi_proxy_msg_logger(LOG_INFO, sock_fd, 0, "new connection from ", hname_buf, NULL);
@@ -969,7 +1001,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
             }
             else
             {  /* neither INET nor named socket -> internal error */
-               sprintf(hname_buf, "%d", peerAddr.sa.sa_family);
+               snprintf(hname_buf, sizeof (hname_buf), "%d", peerAddr.sa.sa_family);
                vbi_proxy_msg_logger(LOG_WARNING, -1, 0, "new connection via unexpected protocol family ", hname_buf, NULL);
             }
          }
@@ -980,7 +1012,7 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
       }
       else
       {  /* socket address buffer too small: internal error */
-         sprintf(hname_buf, "need %d, have %d", length, maxLength);
+         snprintf(hname_buf, sizeof (hname_buf), "need %d, have %d", length, maxLength);
          vbi_proxy_msg_logger(LOG_WARNING, -1, 0, "new connection: saddr buffer too small: ", hname_buf, NULL);
       }
 
@@ -1034,7 +1066,7 @@ static char * vbi_proxy_msg_resolve_symlinks( const char * p_dev_name )
                if (p_tmp2 != NULL)
                {  /* copy former path up to and including the separator character */
                   p_tmp2 += 1;
-                  strncpy(p_tmp, p_path, p_tmp2 - p_path);
+                  strlcpy(p_tmp, p_path, p_tmp2 - p_path);
                }
                else
                {  /* no path separator in the former path -> replace completely */
@@ -1435,3 +1467,10 @@ int vbi_proxy_msg_check_ioctl( VBI_DRIVER_API_REV vbi_api,
 }
 
 #endif  /* ENABLE_PROXY */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
